/**
 * Class responsible for handling interactive events on the graph nodes.
 */
export declare class HandleEvents {
    private currentlyClicked;
    private hostElement;
    /**
     * Creates an instance of HandleEvents.
     *
     * @constructor
     * @param {HTMLElement} hostElement - The HTML element that hosts the D3.js graph.
     */
    constructor(hostElement: any);
    /**
     * Apply mouseover event handling to the nodes for displaying tooltips and highlighting connected nodes.
     *
     * @param {any} nodes - D3 selection of graph nodes.
     * @param {any} links - D3 selection of graph links.
     * @param {any} tooltip - D3 selection of the tooltip element.
     */
    applyMouseover(nodes: any, links: any, tooltip: any): void;
    /**
     * Clears the current selection by resetting node and link styles.
     */
    clearSelection(): void;
    /**
     * Handles the onClick event for nodes, updating their sizes and related links.
     *
     * @param {any} nodes - D3 selection of graph nodes.
     * @param {any} links - D3 selection of graph links.
     */
    onClick(nodes: any, links: any): void;
    handleNodeClick(event: any, d: any, nodes: any, links: any): void;
    /**
     * Applies a click handler to the body element to reset currentlyClicked when clicking outside the graph.
     */
    applyClickHandler(): void;
    /**
     * Applies drag functionality to nodes using D3 drag behavior.
     *
     * @param {any} nodes - D3 selection of graph nodes.
     * @param {any} simulation - D3 force simulation.
     */
    applyDragToNodes(nodes: any, simulation: any): void;
}
